var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import { preparePaginationTC } from './types/preparePaginationType';
/* eslint-disable no-param-reassign, no-use-before-define */

const DEFAULT_PER_PAGE = 20;

export function preparePaginationResolver(tc, opts) {
  if (!tc || tc.constructor.name !== 'TypeComposer') {
    throw new Error('First arg for prepareConnectionResolver() should be instance of TypeComposer');
  }

  if (!opts.countResolverName) {
    throw new Error(`TypeComposer(${tc.getTypeName()}) provided to composeWithConnection ` + 'should have option `opts.countResolverName`.');
  }
  const countResolver = tc.getResolver(opts.countResolverName);
  if (!countResolver) {
    throw new Error(`TypeComposer(${tc.getTypeName()}) provided to composeWithConnection ` + `should have resolver with name '${opts.countResolverName}' ` + 'due opts.countResolverName.');
  }
  const countResolve = countResolver.getResolve();

  if (!opts.findResolverName) {
    throw new Error(`TypeComposer(${tc.getTypeName()}) provided to composeWithConnection ` + 'should have option `opts.findResolverName`.');
  }
  const findManyResolver = tc.getResolver(opts.findResolverName);
  if (!findManyResolver) {
    throw new Error(`TypeComposer(${tc.getTypeName()}) provided to composeWithConnection ` + `should have resolver with name '${opts.findResolverName}' ` + 'due opts.countResolverName.');
  }
  const findManyResolve = findManyResolver.getResolve();

  const additionalArgs = {};
  if (findManyResolver.hasArg('filter')) {
    const filter = findManyResolver.getArg('filter');
    if (filter) {
      additionalArgs.filter = filter;
    }
  }
  if (findManyResolver.hasArg('sort')) {
    const sort = findManyResolver.getArg('sort');
    if (sort) {
      additionalArgs.sort = sort;
    }
  }

  return new tc.constructor.schemaComposer.Resolver({
    type: preparePaginationTC(tc),
    name: 'pagination',
    kind: 'query',
    args: _extends({
      page: {
        type: 'Int',
        description: 'Page number for displaying'
      },
      perPage: {
        type: 'Int',
        description: '',
        defaultValue: opts.perPage || DEFAULT_PER_PAGE
      }
    }, additionalArgs),
    // prettier-ignore
    resolve: async ( /* :: <TContext> */rp /* : $Shape<PaginationResolveParams<TContext>> */
    ) => {
      let countPromise;
      let findManyPromise;
      const { projection = {}, args, rawQuery } = rp;
      const findManyParams /* : $Shape<ResolveParams<any, TContext>> */ = _extends({}, rp);

      const page = parseInt(args.page, 10) || 1;
      if (page <= 0) {
        throw new Error('Argument `page` should be positive number.');
      }
      const perPage = parseInt(args.perPage, 10) || opts.perPage || DEFAULT_PER_PAGE;
      if (perPage <= 0) {
        throw new Error('Argument `perPage` should be positive number.');
      }

      const countParams /* : $Shape<ResolveParams<any, TContext>> */ = _extends({}, rp, {
        rawQuery,
        args: {
          filter: _extends({}, rp.args.filter)
        }
      });

      if (projection.count || projection.pageInfo && (projection.pageInfo.itemCount || projection.pageInfo.pageCount)) {
        countPromise = countResolve(countParams);
      } else {
        countPromise = Promise.resolve(0);
      }

      if (projection && projection.items) {
        // combine top level projection
        // (maybe somebody add additional fields via rp.projection)
        // and items (record needed fields)
        findManyParams.projection = _extends({}, projection, projection.items);
      } else {
        findManyParams.projection = _extends({}, projection);
      }

      const limit = perPage;
      const skip = (page - 1) * perPage;

      findManyParams.args.limit = limit + 1; // +1 document, to check next page presence
      if (skip > 0) {
        findManyParams.args.skip = skip;
      }

      // pass findMany ResolveParams to top resolver
      rp.findManyResolveParams = findManyParams;
      rp.countResolveParams = countParams;

      // This allows to optimize and not actually call the findMany resolver
      // if only the count is projected
      if ((projection.count || projection.pageInfo) && Object.keys(projection).length === 1) {
        findManyPromise = Promise.resolve([]);
      } else {
        findManyPromise = findManyResolve(findManyParams);
      }

      return Promise.all([findManyPromise, countPromise]).then(([items, count]) => {
        const result = {
          count,
          items: items.length > limit ? items.slice(0, limit) : items,
          pageInfo: {
            currentPage: page,
            perPage,
            itemCount: count,
            pageCount: Math.ceil(count / perPage),
            hasPreviousPage: page > 1,
            hasNextPage: items.length > limit || page * perPage < count
          }
        };
        return result;
      });
    }
  });
}